%%The MIT License (MIT)

%Copyright (c) 2014 Harbing

%Permission is hereby granted, free of charge, to any person obtaining a copy
%of this software and associated documentation files (the "Software"), to deal
%in the Software without restriction, including without limitation the rights
%to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
%copies of the Software, and to permit persons to whom the Software is
%furnished to do so, subject to the following conditions:

%The above copyright notice and this permission notice shall be included in
%all copies or substantial portions of the Software.

%THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
%IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
%FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
%AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
%LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
%OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
%THE SOFTWARE.



clear all, close all;



%% Import image file and process with exp processing
uiwait(msgbox('Please select a diffraction .png file'));

    [filename, pathname] = uigetfile('*.png') %read file path and name with standard dialog

    fig1 = imread(strcat(pathname, filename)); %import image file as fig1 after combining pathname and filename together
    
    imshow(fig1)
    
    
%% generate the center of the diffraction pattern. The four point selection and labeling are all identical so I only note on the first one
uiwait(msgbox('Please select four points that are semetric to the center. This is to determine the actual center of the pattern'));
   

    % #1 point
    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped) % Use auto spot function to find spot in the cropped image
    
    center_x1= temp_x + rect(1,1) % calculate the spot position in the original image
    
    center_y1= temp_y + rect(1,2)
    
    hold on                           % plot a mark in green and label with 'C'
    plot(center_x1, center_y1, 'g*')
    text(center_x1+20, center_y1-20, 'C')
    
    
    
    
    % #2 point
    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped)
    
    center_x2= temp_x + rect(1,1)
    
    center_y2= temp_y + rect(1,2)
    
    hold on
    plot(center_x2, center_y2, 'g*')
    text(center_x2+20, center_y2-20, 'C')
    
    
    % #3 point
    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped)
    
    center_x3= temp_x + rect(1,1)
    
    center_y3= temp_y + rect(1,2)
    
    hold on
    plot(center_x3, center_y3, 'g*')
    text(center_x3+20, center_y3-20, 'C')
    
    
    % #4 point
    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped)
    
    center_x4= temp_x + rect(1,1)
    
    center_y4= temp_y + rect(1,2)
    
    hold on
    plot(center_x4, center_y4, 'g*')
    text(center_x4+20, center_y4-20, 'C')
    

    % calculate the center of diffraction pattern
    
     center_x= (center_x1+center_x2+ center_x3+center_x4)/4
     
     center_y= (center_y1+center_y2+ center_y3+center_y4)/4
     
     hold on
     plot(center_x, center_y, 'b*')
     text(center_x+20, center_y+20, 'Center')
     
     
     
%% Specify reference spot 

uiwait(msgbox('Select one reference spot that you know its corresponding d spacing'))

    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped) % use autospot function to find the spot center
    
    reference_x= temp_x + rect(1,1)
    
    reference_y= temp_y + rect(1,2)
    
    hold on
    plot(reference_x, reference_y, 'r*')
    text(reference_x+20, reference_y+20, 'Ref')
    
    input_spot_distance = [center_x, center_y, reference_x, reference_y]
    
    reference_distance= spot_distance(input_spot_distance)
    vector_reference_x = reference_x - center_x
    vector_reference_y = reference_y - center_y

    
    
%% ask user input the reference dspacing information from user
    prompt = {'reference d spacing (nm):'};% Ask user to input the reference d spacing
    dlg_title = 'Input';
    num_lines = 1;
    def = {'0'};
    Ref=inputdlg(prompt,dlg_title,num_lines,def);

    reference_dspacing = str2num(Ref{:})
    
%% Convert image into XRD like data file

[x,y]=size(fig1)% find the size of the two dimensions of fig

diatance_matrix = zeros(x,y);% infact, I don't think x and y are good habbit, they may cause difficulty in the future, should all change to rows and columns

%for i= 1:x
%    for j = 1:y
 %       distance_matrix(i,j)= round(pdist([i,j ; center_x, center_y]));
 %   end
%end

temp_vector_row=(1:x)'; % create a vector that represents the row numbers
row_matrix = repmat(temp_vector_row,1,y); % convert into a matrix with the same size of fig1 but only have the row number

temp_vector_column = (1:y);% create column numbers
column_matrix = repmat(temp_vector_column, x, 1); % create column matrix

row_distance = row_matrix - center_x; % show the row distance in row_matrix
column_distance = column_matrix - center_y; % show the column distance in column_matrix

square_distance = row_distance.^2+column_distance.^2; % calculate the distance in the pythagoras law. The matrix pwoer needs  .^2 
distance_matrix = square_distance.^0.5;

distance_vector = reshape(distance_matrix, [], 1); % change distance_matrix into a vector
counts_vector_invert=reshape(fig1, [], 1); % again, make the counts into a vertor.

counts_vector_flat = 255 - counts_vector_invert; 

counts_vector=counts_vector_flat.^3;

distance_order = 1: round(sqrt(x^2+y^2)); % create a vector to store the distance in order.
counts_order = zeros(size(distance_order)); % make a corresponding vector for storing the counts together with the distance_order

XRD_counts = vertcat(distance_order, counts_order); % combine the counts and distance together
%% 
%for i=1:size(distance_vector) % sorting and adding the counts at the same distance, equivalent to an integration
%    temp_distance = round(distance_vector(i));
%    current_counts = counts_order(temp_distance);
%    counts_order(temp_distance) = current_counts + counts_vector(i);

%end

%distance_order= unique(round(distance_vector)); % find out all unique elements in diatance_vector and show them in ascending order.
%temp_indices = [distance_vector ones(size(distance_vector))]; % create a indices for the distance_vector, but I think it is not so neccesary in accumarray
%counts_order = accumarray(round(temp_indices) , counts_vector);



%% Get study points.

uiwait(msgbox('Please select diffraction spots that you would like to study. You have to click on the image each time before you select. Press any key to stop'))

i=0

while 1
    
    i=i+1
    
    k = waitforbuttonpress
    
    if k==1
        break
    else
    end

    rect = getrect% specify a rectangular with xmin ymin width height
    
    cropped = imcrop(fig1, rect) % crop the image depends on the rect
    
    [temp_x, temp_y]= autospot(cropped) % use autospot function to find the spot center
    
    spot_x= temp_x + rect(1,1) % calculate the center position in the original image from the center of the cropped image
    
    spot_y= temp_y + rect(1,2)
    
    input_spot_distance = [center_x, center_y, spot_x, spot_y]

    study_distance(1,i)= spot_distance(input_spot_distance)% calculate the distance between dots and the Diff center
    
    dspacing(1,i) = reference_dspacing * reference_distance/study_distance(1,i)% calculate dspacing

    vector_x= spot_x - center_x % calculate the vector of each spot

    vector_y= spot_y - center_y

    input_spot_angle = [vector_reference_x, vector_reference_y, vector_x, vector_y] % using the spot_angle function to get the angle in deg
    
    angle_to_reference(1, i) = spot_angle(input_spot_angle) 


    n = num2str(i)
    
    hold on
    plot(spot_x, spot_y, 'b*')
    text(spot_x+20, spot_y+20, n)
   
   
    
   
end
    
char

   


%% print results into a file

expression = '.png'; % replace the .png extenstion
replace = '-dspacing.txt'; % replace extension with .txt

export_filename_txt = strcat(pathname, regexprep(filename,expression,replace)) % inlclude path and file name together to make sure files that outside the Matlab folder can be read

fileID = fopen(export_filename_txt,'w');

fprintf(fileID,'%6s %12s %16s\n','#','dspacing(nm)', 'angle');

for i=1:str2num(n)
    
    str_label=num2str(i)

    str_dspacing=num2str(dspacing(1,i))

    str_angle=num2str(angle_to_reference(1,i))
    
    fprintf(fileID, '%6.2s %12.8s %16.8s\n', str_label, str_dspacing, str_angle)
end



fclose(fileID);